package com.ibm.mqe.bridgemodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import com.ibm.uia.metamodel.*;
import com.ibm.uia.renderingInterface.*;
import com.ibm.mqe.mqbridge.*;
import com.ibm.mqe.model.*;
import java.util.*;

/**
 * Insert the type's description here.
 * Creation date: (12/07/2000 11:44:56)
 * @author: Phill van Leersum
 */
public class MQeBridgeQueueManagerProxy extends QueueManager implements BridgeModelConstants {
	private static final String CMD_CREATE_BRIDGE_QUEUE = "BridgeQueue";
	MQeBridgeListProxy bridgesList;
/**
 * MQeBridgeQueueManagerProxy constructor comment.
 * @param name java.lang.String
 * @param metaObject com.ibm.uia.metamodel.MetaObject
 */
public MQeBridgeQueueManagerProxy() {
	super();
}
protected boolean _createBridgeQueue(Renderer renderer, String[] data) {
	boolean reply = false;
	if (null != data) {
		String type = BRIDGE_QUEUE_TYPE_NAME;
		String name = data[0];
		UIObject uio = findExistingChild(type, name);
		if (null == uio) {
			uio = addNewChild(type, name);
		}
		if (null != uio) {
			MQeBridgeQueueProxy bqp = (MQeBridgeQueueProxy)uio;
			bqp.initialiseFrom(data);
			reply = bqp.create();
		}
	}
	return reply;
}
protected boolean _hasBridgeData(MQeFields data) {
	try {
		MQeFields[] allChildData = data.getFieldsArray(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILDREN);
		for (int i = 0; i < allChildData.length; i++){
			String childName = allChildData[i].getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILD);
		}
	} catch (Exception e) {
		return false;
	}
	return true;
}
/**
 * Read the receivers data from the output data field
 * Overriden here because we get the data for bridges and 
 * queues/connections in separate messages.  We have to ensure 
 * that we do not delete the queue/connections when updating
 * bridge resources.
 * We do this by preventing the normal (super class) processing
 * when handling bridge data.
 */
protected boolean _readRefreshData(MQeFields data) {
	boolean reply = false;
	if (_hasBridgeData(data)) {
		if (null == bridgesList) {
			bridgesList = (MQeBridgeListProxy)findExistingChild(BRIDGES_LIST_TYPE_NAME, "bridges");
			if (null == bridgesList) {
				bridgesList = (MQeBridgeListProxy)addNewChild(BRIDGES_LIST_TYPE_NAME, "bridges");
			}
			bridgesList.setStateCreated();
			reply = true;
		}
		bridgesList.refresh();
	} else {
		reply = super._readRefreshData(data);
	}
	return reply;
}
/**
 * Read the receivers data from the output data field. Place all
 * proxies for discovered resources into the supplied vector. Any
 * proxy not in the found collection will  deleted
 *
 * Overriden here because we get the data for bridges and 
 * queues/connections in separate messages.  We have to ensure 
 * that we do not delete the bridge resources. When updating 
 * queues/connections
 */
protected boolean _readRefreshData(MQeFields data, Vector found) {
	if (null != bridgesList) {
		found.addElement(bridgesList);
	}
	return super._readRefreshData(data, found);
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is required.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	String[] data = op.getData();
	String command = op.getName();
	boolean reply = false;
	if (CMD_CREATE_BRIDGE_QUEUE.equals(command)) {
		reply = _createBridgeQueue(renderer, data);
	} else {
		reply = super.performOperation(op, renderer);
	}
	return reply;
}
/**
 * Initiate the sequence of actions which will result in refreshing the
 * receivers data.
 * Reply true if this results in a structural change.
 *
 * Overridden here to initiate a query on bridge data
 */
public boolean refresh() {
	MQeAdminMsg msg = getQueueManagerStub().queryBridgeData();
	boolean reply = handleMessage(msg);
	if (super.refresh()) {
		reply = true;
	}
	return reply;
}
/**
 * Reply the string representing the type as given by the
 * supplied class name.
 */
protected String typeForClass(String className) {
	if (className.equals("com.ibm.mqe.mqbridge.MQeMQBridgeQueue")) {
		return BRIDGE_QUEUE_TYPE_NAME;
	}
	return super.typeForClass(className);
}
}
