package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.mqe.*;
import com.ibm.uia.metamodel.*;
import com.ibm.uia.renderingInterface.*;
import java.util.*;
import com.ibm.mqe.administration.*;
/**
 * Insert the type's description here.
 * Creation date: (24/07/2000 08:13:42)
 * @author: Phill van Leersum
 */
public class Connection extends ManagedObject {
protected boolean _readRefreshData(MQeFields data, Vector found) {
	boolean reply = super._readRefreshData(data, found);
	try {
		MQeFields[] adaptorsData = data.getFieldsArray(MQeConnectionAdminMsg.Con_Adapters);
		for (int i = 0; i < adaptorsData.length; i++){
			String childName = "Adapter " + i;
			String childType = CONNECTIONADAPTOR_TYPE_NAME;
			UIObject uio = findExistingChild(childType, childName);
			if (null == uio) {
				uio = addNewChild(childType,childName);
			}
			if (null != uio) {
				((ConnectionAdaptor)uio)._readAttributes(adaptorsData[i]);
				found.addElement(uio);
			}
			reply = true;
		}
	} catch (Exception e) {
	}
	return reply;
}
/**
 * Create an alias for the receiver
 */
public final MQeAdminMsg addAlias(String aliasName) {
	MQeConnectionAdminMsg msg = connectionAdminMessage();
	MQeAdminMsg replyMsg = null;
	try {
		msg.addAlias(aliasName);
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		
	}
	return replyMsg;
}
/**
 * reply an admin message for the MQe resource 
 * represented by the receiver.
 */
protected MQeAdminMsg adminMessage() {
	return connectionAdminMessage();
}
/**
 * adminMessage method comment.
 */
protected MQeConnectionAdminMsg connectionAdminMessage() {
	MQeConnectionAdminMsg msg = null;
	try {
		msg = new MQeConnectionAdminMsg(getName());
	} catch (Exception e) {
		// error creating queueAdminMessage
		//e.printStackTrace(_log());
	}
	return msg;
}
/**
 * Read the receivers data from the output data field
 */
public final String[] getAliasData(MQeFields data) {
	String[] alii = null;
	try {
		alii = data.getAsciiArray(MQeConnectionAdminMsg.Con_Aliases);
	} catch (Exception e) {
		_comment("No aliases found");
		//e.printStackTrace(_log());
	}
	return alii;
}
/**
 * commit the receiver and its attributes
 * reply true if any change occurs
 */
protected boolean performCreate() {
	if (isToBeCreated()) {
		setStateCreating();
	}
	MQeConnectionAdminMsg msg = connectionAdminMessage();
	MQeAdminMsg replyMsg = null;
	String route = getRouteAddress();
	try {
		if (null == route || "null".equals(route)) {
			MQeFields mqef = new MQeFields();
			mqef.putAscii(msg.Con_Channel, null);
			msg.create(mqef);
			msg.setName(getName());
		} else {
			msg.create(route, null, null, "DefaultChannel", "new connection");
		}
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		_warning("Failed to create admin connection", e);
	}
	return handleMessage(replyMsg);
}
protected boolean performDelete() {
	super.performDelete();
	MQeConnectionAdminMsg msg = connectionAdminMessage();
	MQeAdminMsg replyMsg = null;
	try {
		msg.delete(new MQeFields());
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		
	}
	return handleMessage(replyMsg); 
}
/**
 * Create an alias for the receiver
 */
public final MQeAdminMsg removeAlias(String aliasName) {
	MQeConnectionAdminMsg msg = connectionAdminMessage();
	MQeAdminMsg replyMsg = null;
	try {
		msg.removeAlias(aliasName);
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		
	}
	return replyMsg;
}
}
