package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import com.ibm.uia.metamodel.*;
import com.ibm.uia.xml.*;
import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import java.awt.Color;
import java.io.*;
import java.util.*;
/**
 * This type was created in VisualAge.
 */
public abstract class MQeObject extends UIObject implements MQeStubMessageListener, MQeMetaModelConstants {
	protected static final String CMD_REFRESH = "Refresh";
	protected static final String CMD_COMMIT = "Commit";
	protected static final String CMD_COMMIT_ALL = "Commit All Changes";
	protected static final String CMD_REVERT = "Undo Changes";
	protected static final String CMD_REVERT_ALL = "Undo All Changes";
	protected static final String CMD_ADDALIAS = "Add Alias";
/**
 */
public MQeObject() {
	super();
}
protected final void _addPendingChildrenTo(Vector found) {
	Enumeration enum = getAllChildren();
	while (enum.hasMoreElements()) {
		Object obj = enum.nextElement();
		if (obj instanceof CreatableObject) {
			CreatableObject mqeo = (CreatableObject)obj;
			if (!found.contains(mqeo) && mqeo.isToBeCreated()) {
				found.addElement(mqeo);
			}
			mqeo._addPendingChildrenTo(found);
		}
	}
}
public final int _getAction(MQeAdminMsg msg) {
	try {
		 return msg.getAction();
	} catch (Exception e) {
		_error("error getting message Action", e);
	}
	return -1;
}
final String _getReason(MQeAdminMsg replyMsg) {
	try {
		return replyMsg.getReason();
	} catch (Exception e) {
	}
	return "";
}
final int _getReasonCode(MQeAdminMsg replyMsg) {
	String reason = _getReason(replyMsg);
	String KEYWORD = "Code=";
	int start = reason.indexOf(KEYWORD);
	if (-1 != start) {
		start += KEYWORD.length();
		int end = reason.indexOf(";", start);
		String str = reason.substring(start, end);
		return Integer.parseInt(str);
	}
	return -1;
}
final int _getResultCode(MQeAdminMsg replyMsg) {
	try {
		return replyMsg.getRC();
	} catch (Exception e) {
	}
	return -1;
}
private final boolean _hasUncommittedAttributeChanges() {
	Enumeration enum = getAllAttributes();
	while (enum.hasMoreElements()) {
		RenderableAttribute ra = (RenderableAttribute)enum.nextElement();
		if (!ra.isCommitted()) {
			return true;
		}
	}
	return false;
}
protected final boolean _isSuccess(MQeAdminMsg replyMsg) {
	return MQeAdminMsg.RC_Success == _getResultCode(replyMsg);
}
/**
 * Read the receivers attributes from the 
 * output data field
 */
protected boolean _readAttributes(MQeFields data) {
	Enumeration enum = getAllAttributes();
	while (enum.hasMoreElements()) {
		Object obj = enum.nextElement();
		MQeAttribute att = (MQeAttribute)obj;
		att.readValueFrom(data);
	}
	return true;
}
/**
 * Read the receivers data from the output data field
 */
protected boolean _readRefreshData(MQeFields data) {
	boolean reply = false;
	_readAttributes(data);
	Vector found = new Vector();
	reply = _readRefreshData(data, found);
	reply = removeChildrenNotInList(found) || reply;
	return reply;
}
protected abstract boolean _readRefreshData(MQeFields data, Vector found);
/**
 * Write the receivers attributes to 
 * the output data field
 */
boolean _writeAttributes(MQeFields data) {
	Enumeration enum = getAllAttributes();
	while (enum.hasMoreElements()) {
		Object obj = enum.nextElement();
		MQeAttribute att = (MQeAttribute)obj;
		att.writeValueInto(data);
	}
	return true;
}
/**
 * This is the method called by a stub when delivering an
 * asynchronous reply message.
 */
public void acceptMessage(MQeAdminMsg msg) {
	if (handleMessage(msg)) {
		getRenderer().redisplay(true);
	}
}
/**
 * Check to see if a good reply was required
 * If not detail the error(s) that occured
 * @return boolean  true if good, or if partly good
 * @param replyMsg reply message to check
 */
public final boolean checkReply(MQeAdminMsg replyMsg) {
	boolean reply = true;
	try {
		// Was a reply received ?
		if (replyMsg == null) {
			//_out().println("..No response received to the request");
			//throw new MQeException("No response message received");
			reply = false;
		} else if (replyMsg.getRC() != MQeAdminMsg.RC_Success) {
			reply = false;
			// If mixed then list each error that occured
			if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
				reply = true;
				MQeFields errors = replyMsg.getErrorFields();
				Enumeration en = errors.fields();
				// process each error
				while (en.hasMoreElements()) {
					String value[];
					String name = (String) en.nextElement();
					value = replyMsg.getFieldInError(name);
					for (int j = 0; j < value.length; j++)
						_comment("Field in error: " + name + "  " + value[j]);
				}
			}
		}
	} catch (Exception e) {
		reply = false;
	}
	return reply;
}
/**
 * If the receiver is connected, then attempt to commit all outstanding
 * changes (creates/deletes/updates).
 */
public boolean commit(boolean recurse) {
	boolean changed = false;
	if (isConnected()) {  // only commit if connected
		if (_hasUncommittedAttributeChanges()) {
			if (update(false)) {
				changed = true;
			}	
		}
		if (recurse) {
			Enumeration enum = getAllChildren();
			while (enum.hasMoreElements()) {
				MQeObject mqeo = (MQeObject) enum.nextElement();
				if (mqeo.commit(recurse)) {
					changed = true;
				}
			}
		}
	}
	return changed;
}
public boolean cut() {
	if (super.cut()) {
		commit(true);
		return true;
	}
	return false;
}
/**
 * edit the values of the receivers attributes.
 *
 * overriden here to add a commit
 */
public boolean edit(Renderer renderer) {
	if (super.edit(renderer)) {
		return commit(false);
	}
	return false;
}
protected QueueManager getQueueManager() {
	UIObject parent = getParent();
	while (null != parent) {
		if (parent instanceof QueueManager) {
			return (QueueManager)parent;
		}
		parent = parent.getParent();
	}
	return null;
}
/**
 * This is the method called to handle all reply messages, whether delivered
 * asynchronously (through the callback method 'acceptMessage') or synchronously
 * as a returned value from a stub operation.
 */
protected boolean handleMessage(MQeAdminMsg msg) {
	boolean changed = false;
	if (null != msg) {
		MQeFields data = msg.getOutputFields();
		switch (_getAction(msg)) {
			case MQeAdminMsg.Action_InquireAll :
				if (_isSuccess(msg)) {
					_readRefreshData(data);
					changed = true;
				} else {
					System.out.println("Inquire failed: " + msg.getReason());
				}
				break;
			case MQeAdminMsg.Action_Update :
				_comment("update reply");
				if (_isSuccess(msg)) {
					_readRefreshData(data);
					changed = true;
				}
				break;
		}
	}
	return changed;
}
protected boolean importFile(String filename, Renderer renderer) {
	if (super.importFile(filename, renderer)) {
		commit(true);
		return true;
	}
	return true;
}
public boolean isConnected() {
	return getQueueManager().isConnected();
}
/**
 * Reply true if the supplied renderable operation is currently enabled 
 * for the receiver.  Reply false if the operation is not currently
 * enabled.
 */
public boolean isOperationEnabled(RenderableOperation rop) {
	boolean reply = true;
	String cmd = rop.getName();
	if (CMD_COMMIT.equals(cmd)) {
		reply = isConnected();
	} else if (CMD_COMMIT_ALL.equals(cmd)) {
		reply = isConnected();
	} else if (CMD_REFRESH.equals(cmd)) {
		reply = isConnected();
	} else {
		reply = super.isOperationEnabled(rop);
	}
	return reply;
}
public boolean paste() {
	boolean pasted = super.paste();
	if (pasted) {
		commit(true);
	}
	return pasted;
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is required.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	String command = op.getName();
	boolean reply = false;
	if (CMD_REFRESH.equals(command)) {
		reply = refresh();
	} else if (CMD_COMMIT.equals(command)) {
		reply = commit(false);
	} else if (CMD_COMMIT_ALL.equals(command)) {
		reply = commit(true);
	} else if (CMD_REVERT.equals(command)) {
		reply = undo(false);
	} else if (CMD_REVERT_ALL.equals(command)) {
		reply = undo(true);
	} else {
		reply = super.performOperation(op, renderer);
	}
	return reply;
}
public final MQeAdminMsg putAdminMessage(MQeAdminMsg msg){
	return getQueueManager().putAdminMessage(msg, this);
}
public boolean refresh() {return false;}
protected boolean undo(boolean recurse) {
	revert();
	if (recurse) {
		Enumeration enum = getAllChildren();
		while (enum.hasMoreElements()) {
			MQeObject mqeo = (MQeObject)enum.nextElement();
			mqeo.undo(recurse);
		}
	}
	return true;
}
boolean update(boolean recurse) {return false;}
}
