package com.ibm.mqe.model;

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import java.io.*;

/**
 * Insert the type's description here.
 * Creation date: (10/9/2000 2:40:46 PM)
 * @author: 
 */
public class QueueManagerWrapper extends MQeAdminUtils {
	private static final String MY_ATTRIBUTE_RULES = "com.ibm.mqe.model.AttributeRule";
	private MQeQueueManager myQM = null;
	private String baseDirectory;

	private String port;
	private String queueManagerName;
/**
 * Wrapper to create queue manager with given name.
 */
public QueueManagerWrapper(String queueManagerName) {
	super();
	new AttributeRule();  // force a dependancy...
	this.queueManagerName = queueManagerName;
}
/**
 * Wrapper to create queue manager with given name.
 * Wrapper to create queue manager with given port.
 */
public QueueManagerWrapper(String queueManagerName, String port) {
	this(queueManagerName);
	this.port = port;
}
/**
 * Create the aliases neccessary for the creation of a queue manager
 *
 * Part of the programatic creation of a queue manager
 *
 */
private final void _createAliases() {
	try {
		MQe.alias("FileRegistry", 		"com.ibm.mqe.registry.MQeFileSession");
		MQe.alias("MsgLog", 			"com.ibm.mqe.adapters.MQeDiskFieldsAdapter");
		MQe.alias("DefaultTransporter", "com.ibm.mqe.MQeTransporter");
		MQe.alias("Network", 			"com.ibm.mqe.adapters.MQeTcpipHttpAdapter");
		MQe.alias("DefaultChannel", 	"com.ibm.mqe.MQeChannel");
	} catch (Exception e) {
		_error("creating aliases ", e);
	}
}
/**
 * defQMRegistry - create the queue manager, ie create the
 * registry definitions. If there is an error, any newly
 * created definitions are deleted so that the registry is
 * left in its initial state.
 *
 * @param iniFields  A MQeFields object containing the configuration parameters.
 * @param queueDir      The directory for the queue store.
 *
 **/
private final boolean _createDefaultRegistry(MQeFields iniFields, String queueDir) {
	boolean success = false;
	// Configuration object
	MQeQueueManagerConfigure qMgrConfig = null;
	try {
		qMgrConfig = new MQeQueueManagerConfigure(iniFields, queueDir);
		qMgrConfig.setChnlAttributeRuleName(MY_ATTRIBUTE_RULES);

		qMgrConfig.defineQueueManager();
		qMgrConfig.defineDefaultAdminQueue();
		qMgrConfig.defineDefaultAdminReplyQueue();
	} catch (Exception e) {
		_error(" creating default registry", e);
	} finally {
			qMgrConfig.close();
	}
	return success;
}
/**
 * Create the mimimum ini fields neccessary for the creation of a queue manager
 *
 * Part of the programatic creation of a queue manager
 *
 * Specify queue manager name, and specify port. (if port is null, then a
 * listener is nt created).
 * the registry is created underneath the curren directory
 */
private final MQeFields _createIniFields(String qmName) {
	
	MQeFields iniFields = new MQeFields();
	try {
		// create registry section
		MQeFields section = new MQeFields();
		section.putAscii( com.ibm.mqe.registry.MQeRegistry.LocalRegType, "FileRegistry");
		section.putAscii( com.ibm.mqe.registry.MQeRegistry.DirName, baseDirectory + "registry");
		iniFields.putFields(MQeQueueManager.Registry, section);

		// create queue manager section
		section = new MQeFields();
		section.putAscii(MQeQueueManager.Name, qmName);
		iniFields.putFields(MQeQueueManager.QueueManager, section);

	} catch (Exception e) {
		_error("creating inifields ", e);
	}
	return iniFields;
}
/**
 * Create the queue manager
 *
 * Part of the programatic creation of a queue manager
 *
 * Specify queue manager name, and specify port. (if port is null, then a
 * listener is nt created).
 * the registry is created underneath the current directory
 *
 * @param iniFields  A MQeFields object containing the configuration parameters.
 */
private final boolean _createQueueManager(MQeFields iniFields) {
	boolean success = false;
	// create directory for queues store
	String queueDir = baseDirectory + "queues" + File.separator;
	boolean queueDirCreated = false;
	File dirFile = new File(queueDir);
	if (!dirFile.exists()) {
		_comment("Creating QueueManager queue directory");
		dirFile.mkdirs();
		queueDirCreated = true;
	}
	success = _createDefaultRegistry(iniFields, "MsgLog:" + queueDir);
	if (!success) {
		File dir = new File(baseDirectory);
		dir.delete();
	}
	return success;
}
/**
 * Open the QueueManager section in the conguration data                  
 * Returns null if the open failed                    
 * @param sections  A MQeFields object containing the configuration parameters.
 */
private final MQeQueueManager _openQueueManager(MQeFields sections, boolean throwExceptions) {
	MQeQueueManager queueManager = new MQeQueueManager();
	try {
		_comment("Opening QueueManager");
		MQeChannelManager channelManager = new MQeChannelManager();
		channelManager.numberOfChannels(0);
		queueManager.setGlobalHashTable(channelManager.getGlobalHashtable());
		queueManager.activate(sections);
		MQeChannelListener channelListener = new MQeChannelListener("Network::" + port, "Network:", channelManager);
		channelListener.setTimer(300);
		_comment("started listener");
	} catch (Exception e) {
		if (throwExceptions) {
			_error("Could not open QueueManager " + getName()+ " :" + e.getMessage());
		}
		queueManager = null; 
	}
	return (queueManager);
}
public final String getName() {
	return queueManagerName;
}
public final String getPort() {
	return port;
}
public final MQeQueueManager getQueueManager() {
	return myQM;
}
public final String getRoute() {
	String route = "no route found";
	try {
		// potential bug here: I cannot find out from MQeQueueManager
		// what port it is listening on:  I have hard coded '8081'
		// untl I figure out the best way to do this
		route = java.net.InetAddress.getLocalHost().getHostAddress()
			+ getPort();
	} catch (java.net.UnknownHostException e) {
	}
	return route;
}
public final boolean shutdown(boolean removeDirectory) {
	try {
		_comment("Closing QM");
		myQM.close();
	} catch (MQeException e) {
		if (e.code() != MQe.Except_QMgr_NotActive) {
			_comment("Shutting down. Exception thrown: ", e);
		}
	} catch (Exception e) {
		_comment("Shutting down. Exception thrown: ", e);
	}
	new File(baseDirectory).delete();
	return true;
}
/**
 * .
 */
public boolean start() {

	String str = "no port";
	if (null != port) {
		str = port;
	}
	_comment("QueueManager " + queueManagerName + " using " + str);

	baseDirectory = new File(".").getAbsolutePath() + File.separator + queueManagerName + File.separator;
	
	boolean success = false;
	MQeFields iniFields = _createIniFields(queueManagerName);
	_createAliases();
	
	myQM = _openQueueManager(iniFields, false);
	if (null != myQM) {
		success = true;
	} else {
		_comment("Could not open queue manager, try creating it");
		_createQueueManager(iniFields);
		myQM = _openQueueManager(iniFields, true);
	}
	if (null != myQM) {
		success = true;
		_comment("..Started queue manager: ");
	}
	return success;
}
}
