package com.ibm.uia.metamodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import java.util.*;
import com.ibm.uia.xml.*;
/**
 * This class implements the behaviour of an MQAttributeType.
 * It acts as a typespace in containing all known type instances.
 *  Since there is only ever one typespace, this makes a handy
 * (package)global to keep it in. 
 */
public abstract class AttributeType extends ModelComponent implements RenderableAttributeType {
	protected static final String ATTRIBUTE_TYPE = "attributeType";
	protected static final String ATTRIBUTE_TYPE_MODE = "mode";
	protected static final String ATTRIBUTE_TYPE_DEFAULT = "defaultValue";
	protected static final String ATTRIBUTE_TYPE_DEFAULT_VALUE = "caption";
	protected static final String ENUMERATION_MODE = "enumeration";
	protected static final String STRING_MODE = "string";
	protected static final String INTEGER_MODE = "integer";
	
	private MetaModel metaModel;	
	protected String defaultValue;
/**
 * AttributeType constructor comment.
 * Note that we remove the dom-attributes we use, so that
 * subclasses can ignore them
 */
public AttributeType(MetaModel metaModel) {
	super();
	this.metaModel = metaModel;
}
/**
 * create a DOM representation of the receiver using
 * the supplied document.  The Element generated should
 * be returned to the caller, and not appended to the 
 * document.  Subclasses can override this method but
 * are strongly recommended to call the superclass
 * method FIRST and decorate the returned element with their
 * specific data.
 * Note that the superclass is responsible for navigating
 * the standard tree structures, so this responsibility
 * is removed from subclass implementors
 */
protected Element generateDom(Document doc) {
	Element el = super.generateDom(doc);
	
	Element child = doc.createElement(ATTRIBUTE_TYPE_DEFAULT);
	child.setAttribute(ATTRIBUTE_TYPE_DEFAULT_VALUE, getDefaultValue());
	el.appendChild(child);
	
	return el;
}
/**
 * getDefaultValue method comment.
 */
public String getDefaultValue() {
	return defaultValue;
}
/**
 * reply the data value to place on data stream.
 */
public String getMappedValue(String str) {
	return str;
}
public final MetaModel getMetaModel() {
	return metaModel;
}
/**
 * Reply the string used to identify an XML tag as representing
 * an instance of the receivers class
 */
public final String getTagName() {
	return ATTRIBUTE_TYPE;
}
/**
 * getValidValues method comment.
 */
public Enumeration getValidValues() {
	return null;
}
/**
 * isEnumeratedType method comment.
 */
public boolean isEnumeratedType() {
	return false;
}
/**
 * isIntegerType method comment.
 */
public boolean isIntegerType() {
	return false;
}
/**
 * A convenience ethod for the benefit of the editor:- it
 * allows editor to keep track of type usage.  The return of
 * 'true is completely meaningless here.
 */
public boolean isInUse() {
	return true;
}
/**
 * isStringType method comment.
 */
public boolean isStringType() {
	return false;
}
/**
 * isValid method comment.
 */
public boolean isValid(String str) {
	return false;
}
/**
 * Initialise a child of the receiver from the given Element.
 */ 
protected UIAComponent parseChildElement(Element element) {
	String elemName = element.getNodeName();
	if (ATTRIBUTE_TYPE_DEFAULT.equals(elemName)) {
		defaultValue = element.getAttribute(ATTRIBUTE_TYPE_DEFAULT_VALUE);
	} else {
		return super.parseChildElement(element);
	}
	return null;
}
/**
 * A convenience ethod for the benefit of the editor:- it
 * allows editor to keep track of type usage
 */
protected void registerUseBy(AttributeDefinition attDef, boolean used) {
}
/**
 * reply UI value mapped to the given datastream value.
 */
public String unmap(String str) {
	return str;
}
/**
 * validate method comment.
 */
public String validate(String str) {
	if (isValid(str)) {
		return str;
	}
	return getDefaultValue();
}
}
