package com.ibm.uia.metamodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.util.*;
import com.ibm.uia.xml.*;
/**
 * This type was created in VisualAge.
 */
public class EnumeratedType extends AttributeType {
	public static final String ENUMERATION = "enumeration";
	public static final String LITERAL = "literal";
	public static final String CAPTION_VALUE = "caption";
	public static final String MAPPED_VALUE = "mappedValue";

	private static final String NO_MAPPED_VALUE = "_none_";
	// values and their maped values if any.  Key is value,
	// value is either mapped value, or the constant NO_MAPPED_VALUE
	protected Hashtable valueMap;
/**
 * EnumeratedType constructor comment.
 */
public EnumeratedType(MetaModel metaModel) {
	super(metaModel);
	valueMap = new Hashtable();
}
private final void _parseEnumerationElem(Element elem) {
	valueMap = new Hashtable();
	Enumeration enum = elem.getChildren();
	while (enum.hasMoreElements()) {
		Element child = (Element)enum.nextElement();
		String elemName = child.getNodeName();
		if (LITERAL.equals(elemName)) {
			String key = child.getAttribute(CAPTION_VALUE);
			String val = child.getAttribute(MAPPED_VALUE);
			if (null == val) {
				val = NO_MAPPED_VALUE;
			}
			valueMap.put(key.trim(), val.trim());
			if (null == defaultValue) {
				defaultValue = key;
			}
		}
	}
}
/**
 * create a DOM representation of the receiver using
 * the supplied document.  The Element generated should
 * be returned to the caller, and not appended to the 
 * document.  Subclasses can override this method but
 * are strongly recommended to call the superclass
 * method FIRST and decorate the returned element with their
 * specific data.
 * Note that the superclass is responsible for navigating
 * the standard tree structures, so this responsibility
 * is removed from subclass implementors
 */
protected Element generateDom(Document doc) {
	Element el = super.generateDom(doc);
	el.setAttribute(ATTRIBUTE_TYPE_MODE, ENUMERATION_MODE);
	Element enumEl = doc.createElement(ENUMERATION);
	el.appendChild(enumEl);
	
	Enumeration enum = valueMap.keys();
	while (enum.hasMoreElements()) {
		Element child = doc.createElement(LITERAL);
		String key = enum.nextElement().toString();
		String val = valueMap.get(key).toString();
		child.setAttribute(CAPTION_VALUE, key);
		if (!val.equals(NO_MAPPED_VALUE)) {
			child.setAttribute(MAPPED_VALUE, val);
		}
		enumEl.appendChild(child);
	}
	return el;
}
/**
 * reply the data value to place on data stream.
 */
public String getMappedValue(String str) {
	return valueMap.get(str).toString();
}
/**
 * getValidValues method comment.
 */
public final Enumeration getValidValues() {
	return valueMap.keys();
}
/**
 * isEnumeratedType method comment.
 */
public final boolean isEnumeratedType() {
	return true;
}
/**
 * isValid method comment.
 */
public final boolean isValid(String str) {
	return valueMap.containsKey(str);
}
/**
 * Initialise a child of the receiver from the given Element.
 */ 
protected final UIAComponent parseChildElement(Element elem) {
	String elemName = elem.getNodeName();
	if (ENUMERATION.equals(elemName)) {
		_parseEnumerationElem(elem);
	} else {
		super.parseChildElement(elem);
	}
	return this;
}
/**
 * return UI value mapped to given datastream value.
 */
public String unmap(String str) {
	Enumeration enum = valueMap.keys();
	while (enum.hasMoreElements()) {
		String key = enum.nextElement().toString();
		String val = valueMap.get(key).toString();
		if (val.equals(str)) {
			return key;
		}
	}
	return str;
}
/**
 * validate method comment.
 */
public String validate(String str) {
	if (isValid(str)) {
		return str;
	}
	return getDefaultValue();
}
}
