package com.ibm.uia.metamodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import com.ibm.uia.xml.*;
import java.util.*;
/**
 * This type was created in VisualAge.
 */
public class Operation extends ModelComponent implements RenderableOperation {
	
	public static final String OPERATION_TAG_TEXT = "operation";
	private static final String PATH = "path";
	private static final String ARGUMENT = "argument";
	private static final String ARGUMENT_VALUE = "value";
	
	// the definition with all the common data
	private OperationDefinition definition;
	// the owner of this operation (the object for which the operation was created)
	private UIObject owner;
	// the data (if any) as paremeters to the operation
	private Vector data;
/**
 * OperationDefinition constructor comment.
 */
protected Operation(UIObject owner, OperationDefinition definition) {
	super();
	this.owner = owner;
	this.definition = definition;
}
/**
 * create a DOM representation of the receiver using
 * the supplied document.  The Element generated should
 * be returned to the caller, and not appended to the 
 * document.  Subclasses can override this method but
 * are strongly recommended to call the superclass
 * method FIRST and decorate the returned element with their
 * specific data.
 * Note that the superclass is responsible for navigating
 * the standard tree structures, so this responsibility
 * is removed from subclass implementors
 */
protected Element generateDom(Document doc) {
	Element element = super.generateDom(doc);
	/*if (null != promptType) {
		Element promptEl = doc.createElement(promptType);
		promptEl.setAttribute(PROMPT_CAPTION, getPromptTitle());
		Enumeration enum = promptValues.elements();
		while (enum.hasMoreElements()) {
			String[] strs = (String[])enum.nextElement();
			Element el = doc.createElement(PROMPT_ELEMENT);
			el.setAttribute(DATALABEL, strs[0]);
			el.setAttribute(DATADEFAULT, strs[1]);
			promptEl.addChild(el);
		}
		element.addChild(promptEl);
	}*/
	return element;
}
public String getCaptionText() {
	return getDefinition().getCaptionText();
}
public final String[] getData() {
	String[] strs = null;
	if (null != data) {
		strs = new String[data.size()];
		for (int i = 0; i < data.size(); i++) {
			strs[i] = data.elementAt(i).toString();
		}
	}	
	return strs;
}
private OperationDefinition getDefinition() {
	return definition; 
}
/**
 * getDisplayPath method comment.
 */
public Enumeration getDisplayPath() {
	return getDefinition().getDisplayPath();
}
public final MetaModel getMetaModel() {
	return owner.getMetaModel();
}
public String getName() {
	return getDefinition().getName();
}
/**
 * Reply the string used to identify an XML tag as representing
 * an instance of the receivers class
 */
public final String getTagName() {
	return OPERATION_TAG_TEXT;
}
public final boolean isEnabled() {
	return owner.isOperationEnabled(this);
}
public boolean isToggle() {
	return false;
}
/**
 * Initialise a child of the receiver from the given Element.
 */ 
protected UIAComponent parseChildElement(Element element) {
	UIAComponent uiac = null;
	if (ARGUMENT.equals(element.getNodeName())) {
		if (null == data) {
			data = new Vector();
		}
		data.addElement(element.getAttribute(ARGUMENT_VALUE));
	} else {
		uiac =  super.parseChildElement(element);
	}
	return uiac;
}
/**
 * Initialise the receiver from the given Element.
 */ 
protected void parseDom(Element element) {
	data = null;
	super.parseDom(element);
}
public final boolean perform(Renderer renderer) {
	boolean reply = false;
	if (getDefinition().hasPrompt()) {
		String[] strs = getDefinition().perform(renderer);
		if (null != strs) {
			data = new Vector();
			for (int i = 0; i < strs.length; i++){
				data.addElement(strs[i]);
			}
			reply = owner.performOperation(this, renderer);
		}
	} else {
		data = null;
		reply = owner.performOperation(this, renderer);
	}
	return reply;
}
}
