package com.ibm.uia.metamodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import java.util.*;
import com.ibm.uia.xml.*;
import java.awt.datatransfer.*;
/**
 * This type is the root for all real UIObjects.
 * This type provides default behaviour for al UIObjects:
 *	-	conversion to and from XML
 *	-	access to attributes
 *	-	access to operations
 *	-	access to children
 *	-	creation of children
 *	-	default operation implementations (Cut, Copy, Paste, Delete, Edit Attributes)
 *
 * Properly speaking this is part of the MODEL rather than the metamodel.
 *
 * Instances of this class and its subclasses must be creatable by a default (no
 * argument) constructor.  Initialisation can be performed in an overide of
 * initialise(String name, MetaObject metaObject) which is guaranteed to be called.
 */
public class UIObject extends ModelComponent implements RenderableObject {
	protected static final String OBJECT_NAME = "name";
	protected static final String OBJECT_ID = "id";
	protected static final String CMD_DELETE = "Delete";
	private static final String CMD_COPY = "Copy";
	private static final String CMD_CUT = "Cut";
	private static final String CMD_PASTE = "Paste";
	private static final String CMD_PASTE_FILE = "Paste File";
	private static final String CMD_EDIT_ATTRIBUTES = "Edit";
	private static final String CMD_OPEN_NEW_RENDERER = "Open";
	private static final String CMD_IMPORT_XML = "Import XML";
	private static final String CMD_EXPORT_XML = "Export XML";
	
	// backpointer to metaObject
	private MetaObject metaObject;
	// the children of this object
	private Vector children;
	// attribute table
	private Hashtable attributes;
	private UIObject parent;
	private String id;
	private Renderer renderer;
/**
 * MetaObject constructor comment.
 */
protected UIObject() {
	super();
	children = new Vector();
}
private final void _addCategories() {
	Enumeration enum = metaObject.getAllChildTypes();
	while (enum.hasMoreElements()) {
		String childType = enum.nextElement().toString();
		MetaObject moChild = getMetaModel().getMetaObject(childType);
		if (null != moChild && moChild.isCategory() && moChild.isAutoAdd()) {
			addNewChild(childType, moChild.getCaptionText());
		}
	}
}
/* search for a child of the receiver that can hold
 * the given uiObject type.  If one is found return it,
 * otherwise return null;
 */
private final UIObject _categoryFor(String type) {
	Enumeration enum = getAllChildren();
	while (enum.hasMoreElements()) {
		UIObject child = (UIObject)enum.nextElement();
		if (child.isValidChildType(type)) {
			return child;
		}
	}
	return null;
}
public final void _comment(String s) {
	Renderer renderer = getRenderer();
	if (null != renderer) {
		renderer.comment(s);
	}
}
public final void _comment(String s, Exception e) {
	_comment(s + " : " + e.getMessage());
}
public final void _error(String s) {
	Renderer renderer = getRenderer();
	if (null != renderer) {
		renderer.error(s);
	}
}
public final void _error(String s, Exception e) {
	_error(s + " : " + e.getMessage());
}
protected final void _setAttributeEditable(String attrName, boolean edit) {
	RenderableAttribute ra = getAttribute(attrName);
	if (null != ra) {
		ra.setEditable(edit);
	}
}
/**
 */ 
final void _setMetaObject(MetaObject mo) {
	metaObject = mo;
}
/**
 */ 
final void _setName(String str) {
	super.setName(str);
	setAttribute(OBJECT_NAME,str);
	setAttribute("Name",str);
}
public void _warning(String s) {
	Renderer renderer = getRenderer();
	if (null != renderer) {
		renderer.warning(s);
	}
}
public final void _warning(String s, Exception e) {
	_warning(s + " : " + e.getMessage());
}
/**
 * performOperation method comment.
 */
protected void addNewChild(UIObject uio) {
	if (null != uio && isValidChildType(uio.getType())) {
		children.addElement(uio);
		uio.setParent(this);
	}
}
/**
 * performOperation method comment.
 */
public UIObject addNewChild(String type, String name) {
	UIObject tobj = null;
	// check for categories which contain this type
	UIObject category = _categoryFor(type);
	if (null != category) {
		tobj = category.addNewChild(type, name);
	} else {
		MetaObject mo = getMetaModel().getMetaObject(type);
		if (null != mo) {
			tobj = mo.createNewObject(name);
		}
		if (null != tobj) {
			children.addElement(tobj);
			tobj.setParent(this);
		}
	}
	return tobj;
}
protected boolean canPaste() {
	boolean canPaste = false;
	Clipboard cb = java.awt.Toolkit.getDefaultToolkit().getSystemClipboard();
	StringSelection ss = new StringSelection("");
	Transferable tr = cb.getContents(ss);
	if (null != tr) {
		try {
			Object xml = tr.getTransferData(DataFlavor.stringFlavor);
			if (null != xml) {
				Element el = Parser.FirstTag(xml.toString());
				canPaste = canPaste(el);
			}
		} catch (UnsupportedFlavorException e) {
			// we cannot handle the data
		} catch (java.io.IOException e) {
			// we cannot handle the data
		}
	}
	return canPaste;
}
protected boolean canPaste(Element element) {
	boolean canPaste = false;
	if (null != element) {
		String pasteType = element.getName();
		canPaste = getType().equals(pasteType)
					|| isValidChildType(pasteType);
	}
	return canPaste;
}
public boolean copy() {
	Clipboard cb = java.awt.Toolkit.getDefaultToolkit().getSystemClipboard();
	StringSelection ss = new StringSelection(toString());
	cb.setContents(ss, ss);
	return false;
}
public boolean cut() {
	copy();
	return delete();
}
public boolean delete() {
	getParent().removeChild(this);
	return true;
}
protected final void displayMessage(String title, String message) {
	getRenderer().displayMessage(title, message);
}
/**
 * edit the values of the receivers attributes.
 */
public boolean edit(Renderer renderer) {
	return renderer.edit(this, false);
}
protected boolean exportFile(String filename, Renderer renderer) {
	boolean success = false;
	Document doc = new Document();
	Element elem = generateDom(doc);
	success = Unparser.UnparseToFile(elem, filename, true );
	return success;
}
protected UIObject findExistingChild(String type, String name) {
	UIObject category = _categoryFor(type);
	if (null != category) {
		return category.findExistingChild(type, name);
	} else {
		Enumeration enum = getAllChildren();
		while (enum.hasMoreElements()) {
			Object obj = enum.nextElement();
			UIObject uio = (UIObject) obj;
			if (uio.matches(name, type)) {
				return uio;
			}
		}
	}
	return null;
}
/**
 * create a DOM representation of the receiver using
 * the supplied document.  The Element generated should
 * be returned to the caller, and not appended to the 
 * document.  Subclasses can override this method but
 * are strongly recommended to call the superclass
 * method FIRST and decorate the returned element with their
 * specific data.
 * Note that the superclass is responsible for navigating
 * the standard tree structures, so this responsibility
 * is removed from subclass implementors
 */
protected Element generateDom(Document doc) {
	Element el = super.generateDom(doc);
	el.setAttribute(OBJECT_NAME, getName());
	el.setAttribute(OBJECT_ID, id);
	Enumeration enum = getAllChildren();
	while (enum.hasMoreElements()) {
		UIObject de = (UIObject)enum.nextElement();
		Element child = de.generateDom(doc);
		el.appendChild(child);
	}
	enum = getAllAttributes();
	while (enum.hasMoreElements()) {
		Attribute de = (Attribute)enum.nextElement();
		Element child = de.generateDom(doc);
		el.appendChild(child);
	}
	return el;
}
/**
 * getAllAttributes method comment.
 */
public Enumeration getAllAttributes() {
	return attributes.elements();
}
/**
 * getAllChildren method comment.
 */
public Enumeration getAllChildren() {
	return children.elements();
}
/**
 * getAllOperations method comment.
 */
public Enumeration getAllOperations() {
	Vector operations = new Vector();	// should we cache this as an instance variable?
	Enumeration enum = metaObject.getAllOperations();
	while (enum.hasMoreElements()) {
		OperationDefinition opDef = (OperationDefinition)enum.nextElement();
		Operation op = new Operation(this, opDef);
		operations.addElement(op);
	}
	return operations.elements();
}
/**
 * getAttribute method comment.
 */
public Attribute getAttribute(String attributeName) {
	return (Attribute)attributes.get(attributeName);
}
/**
 * getAttributeValue method comment.
 */
public String getAttributeValue(String attributeName) {
	Attribute att = getAttribute(attributeName);
	if (null != att) {
		return att.getValue();
	}
	return null;
}
public java.awt.Image getCaptionImage() {
	return getMetaObject().getCaptionImage();
}
public String getCaptionText() {
	return getName();
}
public java.awt.Color getDisplayColour() {
	return java.awt.Color.black;
}
public String getId() {
	return id;
}
public MetaModel getMetaModel() {
	return getMetaObject().getMetaModel();
}
public final MetaObject getMetaObject() {
	return metaObject;
}
/**
 * getName method comment.
 */
public String getName() {
	String n = getAttributeValue(OBJECT_NAME);
	if (null == n) {
		n = super.getName();
	}
	return n;
}
public final UIObject getParent() {
	return parent;
}
protected Renderer getRenderer() {
	if (null == renderer) {
		if (null != parent) {
			renderer = parent.getRenderer();
		}
	}
	return renderer;
}
public UIObject getRootObject() {
	if (null != parent) {
		return parent.getRootObject();
	}
	return null;
}
/**
 * Reply the string used to identify an XML tag as representing
 * an instance of the receivers class.
 * Overriden here to use the receivers nominal type as the tag text
 */
public final String getTagName() {
	return getType();
}
/**
 * getDisplayName method comment.
 */
public final String getType() {
	if (null != metaObject) {
		return metaObject.getName();
	}
	return null;
}
protected boolean importFile(String filename, Renderer renderer) {
	boolean success = false;
	Document doc = Parser.ParseFile(filename);
	if (null == doc) {
		renderer.error("File: " + filename + " is not a valid file");
	} else {
		Element elem = doc.getDocumentElement();
		if (!canPaste(elem)) {
			renderer.error("File: " + filename + " is not a valid import file for " + getType());
		} else {
			success = pasteElement(elem);
		}
	}
	return success;
}
/**
 * Object initialiser.
 */
protected void initialise(String name, MetaObject metaObject) {
	this.metaObject = metaObject;
	this.attributes = metaObject.newAttributeDictionary(this);
	_addCategories();
	id = Integer.toString(hashCode());
	_setName(name);
	getMetaModel()._register(this);
	_setAttributeEditable(OBJECT_NAME, true);
}
public final boolean isCategory() {
	return metaObject.isCategory();
}
/**
 * Reply true if the supplied renderable operation is currently enabled 
 * for the receiver.  Reply false if the operation is not currently
 * enabled.
 */
public boolean isOperationEnabled(RenderableOperation rop) {
	boolean reply = true;
	String cmd = rop.getName();
	if (CMD_PASTE.equals(cmd)) {
		reply = canPaste();
	}
	return reply;
}
/**
 * Reply true if the supplied renderable operation is currently ticked 
 * for the receiver.  Reply false if the operation is not currently
 * tickeded.
 */
public boolean isOperationTicked(RenderableOperation rop) {
	return true;
}
public final boolean isValidChildType(String type) {
	if (null != metaObject) {
		return metaObject.isValidChildType(type);
	}
	return false;
}
protected boolean matches(String name, String type) {
	return name.equals(getName()) && type.equals(getType());
}
/**
 * Initialise a child of the receiver from the given Element.
 */ 
protected UIAComponent parseChildElement(Element element) {
	UIAComponent tobj = null;
	String elemName = element.getNodeName();
	if (elemName.equals(Attribute.ATTRIBUTE_TAG)) {
		String attName = element.getAttribute(Attribute.ATTRIBUTE_NAME);
		String attValue = element.getAttribute(Attribute.ATTRIBUTE_VALUE);
		setAttribute(attName, attValue);
	} else if (Operation.OPERATION_TAG_TEXT.equals(elemName)) {
		String opName = element.getAttribute(Operation.NAME_TAG);
		OperationDefinition opDef = metaObject.getOperationDefinition(opName);
		if (null != opDef) {
			Operation op = new Operation(this, opDef);
			op.parseDom(element);
			if (performOperation(op, getRenderer())) {
				getRenderer().redisplay(true);
			}
		}
	} else if (isValidChildType(elemName)) {
		String childName = element.getAttribute(OBJECT_NAME);
		tobj = findExistingChild(elemName, childName);
		if (null == tobj) {
			tobj = addNewChild(elemName, childName);
		}
		if (null != tobj) {
			tobj.parseDom(element);
		}
	} else {
		tobj = super.parseChildElement(element);
	}
	return tobj;
}
/**
 * Initialise the receiver from the given Element.
 */ 
protected void parseDom(Element element) {
	if (null == children) {
		children = new Vector();
	}
	_setName(element.getAttribute(OBJECT_NAME));
	element.removeAttribute(OBJECT_NAME);
	id = element.getAttribute(OBJECT_ID);
	element.removeAttribute(OBJECT_ID);
	if (null == id || id.length() < 1) {
		id = Integer.toString(hashCode());
	}
	super.parseDom(element);
}
public boolean paste() {
	boolean pasted = false;
	Clipboard cb = java.awt.Toolkit.getDefaultToolkit().getSystemClipboard();
	StringSelection ss = new StringSelection("");
	Transferable tr = cb.getContents(ss);
	if (null != tr) {
		try {
			Object xml = tr.getTransferData(DataFlavor.stringFlavor);
			if (null != xml) {
				Document doc = Parser.Parse(xml.toString());
				Element el = doc.getDocumentElement();
				if (null == el) {
					displayMessage("Error", "Invalid clipboard data");
				} else {
					pasted = pasteElement(el);
				}
			}
		} catch (UnsupportedFlavorException e) {
			_warning("Paste failed - recognised data type");
		} catch (java.io.IOException e) {
			_warning("Paste failed - data error");
		}
	}
	return pasted;
}
private final boolean pasteElement(Element element) {
	boolean pasted = false;
	// element MUST be non-null
	String pasteType = element.getName();
	if (isValidChildType(pasteType)) {
		parseChildElement(element);
		pasted = true;
	} else if (getType().equals(pasteType)) {
		parseDom(element);
		pasted = true;
	} else {
		_warning(getType() + " invalid paste type " + pasteType);
	}
	return pasted;
}
public boolean pasteFile() {
	boolean pasted = false;
	return pasted;
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 * Reply true if a stuctural refresh is reqired.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	String[] data = op.getData();
	if (isCategory() && null != parent) {	// categories reflect action to parent.
		return getParent().performOperation(op, renderer);
	}
	boolean reply = false;
	String command = op.getName();
	if (CMD_EDIT_ATTRIBUTES.equals(command)) {
		reply = edit(renderer);
	} else if (CMD_DELETE.equals(command)) {
		reply = delete();
	} else if (CMD_CUT.equals(command)) {
		reply = cut();
	} else if (CMD_COPY.equals(command)) {
		reply = copy();
	} else if (CMD_PASTE.equals(command)) {
		reply = paste();
	} else if (CMD_PASTE_FILE.equals(command)) {
		reply = pasteFile();
	} else if (CMD_IMPORT_XML.equals(command)) {
		if (null != data && null != data[0]) {
			reply = importFile(data[0], renderer);
		}
	} else if (CMD_EXPORT_XML.equals(command)) {
		if (null != data && null != data[0]) {
			reply = exportFile(data[0], renderer);
		}
	}	
	return reply;
}
public void removeAllChildren(String type) {
	UIObject category = _categoryFor(type);
	if (null != category) {
		category.removeAllChildren(type);
	} else {
		Vector v = new Vector();
		Enumeration enum = children.elements();
		while (enum.hasMoreElements()) {
			UIObject uio = (UIObject)enum.nextElement();
			if (!uio.getType().equals(type)) {
				v.addElement(uio);
			}
			children = v;
		}
	}
}
public void removeChild(UIObject uio) {
	UIObject category = _categoryFor(uio.getType());
	if (null != category) {
		category.removeChild(uio);
	} else {
		children.removeElement(uio);
	}
}
/* remove all children of the receiver that do not appear in the list
 * Categories are NOT removed, but ARE scanned for children to remove.
 */
public final boolean removeChildrenNotInList(Vector keep) {
	boolean changed = false;
	Vector remove = new Vector();
	Enumeration children = getAllChildren();
	while (children.hasMoreElements()) {
		UIObject uio = (UIObject)children.nextElement();
		if (uio.isCategory()) {
			changed = uio.removeChildrenNotInList(keep);
		} else if (!keep.contains(uio)) {
			remove.addElement(uio);
		}
	}
	changed = changed || !remove.isEmpty();
	Enumeration discard = remove.elements();
	while (discard.hasMoreElements()) {
		removeChild((UIObject)discard.nextElement());
	}
	return changed;
}
/**
 * reset the receivers internal value to the committed
 * value.
 */
public void revert() {
	Enumeration enum = getAllAttributes();
	while (enum.hasMoreElements()) {
		Attribute attr = (Attribute)enum.nextElement();
		attr.revert();
	}
}
/**
 * setAttribute method comment.
 */
public boolean setAttribute(String name, long value) {
	return setAttribute(name, Long.toString(value));
}
/**
 * setAttribute method comment.
 */
public boolean setAttribute(String name, String value) {
	Attribute att = getAttribute(name);
	if (null != att) {
		att.setValue(value);
		return true;
	}
	return false;
}
final void setParent(UIObject parent) {
	this.parent = parent;
}
/**
 * Set the renderer used by the receiver.  
 * This is useful for asynchronous updates (ie those NOT generated 
 * through the renderer).
 * This method is only guaranteed to be called for UIObjects initially
 * rendered by the renderer, and not for the children of the UIObject.
 *
 * This will set the renderer known to the metamodel.
 */
public void setRenderer(Renderer renderer) {
	getMetaModel().setRenderer(renderer);
	this.renderer = renderer;
}
}
