package com.ibm.uia.renderer;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import com.ibm.uia.renderingInterface.*;
import java.awt.image.*;

/**
 * This type was created in VisualAge.
 */
public abstract class RendererNode {
	// distance between top of cell and top of text, 
	// and  distance between bottom of cell and bottom of text, 
	private static final int VERT_MARGIN= 4;
	// distance between image and text.
	private static final int MARGIN= 5;
	// the selection state of the node
	protected boolean selected = false;
	// the panel owning the node	
	//protected ComponentPanel refreshPanel;


	/**
	 * The rectangle bounding the receivers image
	 */
	private Rectangle imageRect;
	/**
	 * The rectangle bounding the receivers text
	 */
	private Rectangle textRect;
public RendererNode() {
	super();
	textRect = new Rectangle();
	imageRect = new Rectangle();
}
/**
 * calculate width & height of image to be drawn onto the graphics provided.
 */
protected void _calculateImageSize(Graphics g, ImageObserver refreshPanel) {
	Image image = _getImage();
	imageRect.width = 0;
	imageRect.height = 0;
	
	if (null != image) {
		int imHeight = image.getHeight(refreshPanel);
		if (-1 != imHeight) {
			imageRect.height = imHeight;
		}
		int imWidth = image.getWidth(refreshPanel);
		if (-1 != imWidth) {
			imageRect.width = imWidth;
		}
	}
}
/**
 * calculate maximum height based on all heights.
 */
protected int _calculateMaxHeight() {
	int maxHeight = 0;
	if (imageRect.height > maxHeight) {
		maxHeight = imageRect.height;
	}
	if (textRect.height > maxHeight) {
		maxHeight = textRect.height;
	}
	return maxHeight;
}
/**
 * calculate width & height of text to be drawn onto the graphics provided.
 */
protected void _calculateTextSize(Graphics g) {
	String name = _getText();
	if (null != name) {
		FontMetrics fontMetrics = g.getFontMetrics();
		textRect.width = fontMetrics.stringWidth(name);
		textRect.height = fontMetrics.getMaxAscent() + fontMetrics.getMaxDescent();
	} else {
		textRect.width = 0;
		textRect.height = 0;
	}
}
protected final void _comment(String s) {
	//System.out.println(s);
}
/**
 * draw the receivers image onto the graphics provided.
 * the coordinates of he image rectangle (x & y) should
 * already have been set.
 */
protected void _drawImage(Graphics g) {
	Image image = _getImage();
	if (null != image) {
		g.drawImage(image, imageRect.x, imageRect.y, null);
	}
}
/**
 * draw the receivers text onto the graphics provided.
 * the coordinates of the text rectangle (x & y) should
 * already have been set.
 */
protected void _drawText(Graphics g) {
	String name = _getText();
	if (null != name) {
		Color oldColour = g.getColor();
		if (selected) {
			g.setColor(Color.lightGray);
			g.fillRect(textRect.x, textRect.y, textRect.width, textRect.height);
		}
		int off = g.getFontMetrics().getMaxAscent();
		g.setColor(_getDisplayColour());
		g.drawString(name, textRect.x, textRect.y + off);
		g.setColor(oldColour);
	}
}
/**
 * reply the colour for rendering the text
 */
protected Color _getDisplayColour() {
	return Color.black;
}
protected abstract Image _getImage();
protected abstract String _getText();
/**
 * reply the x coordinate of the left hand side of the
 * image
 */
protected int _imageLeft(boolean isTree) {
	return 0;
}
void addVisibleChildrenTo(Vector v) {
	v.addElement(this);
}
public boolean copy() {return false;}
public boolean cut() {return false;}
boolean delete() {
	return false;
}
/**
 * draw the receiver onto the graphics provided.
 * return the y coordinate of the bottom of the last element drawn.
 */
public int draw(Graphics g, int y, boolean isTree, ImageObserver refreshPanel) {
	_calculateImageSize(g, refreshPanel);
	_calculateTextSize(g);

	// calculate maximum height based on all heights.
	int maxHeight = _calculateMaxHeight();
	// add on our space above and below
	maxHeight += 2 * VERT_MARGIN;
	int hMid = y + (maxHeight /2);
	imageRect.x = _imageLeft(isTree);
	imageRect.y = hMid - (imageRect.height / 2);

	textRect.x = imageRect.x + imageRect.width + MARGIN;
	textRect.y = hMid - (textRect.height / 2);

	_drawImage(g);
	_drawText(g);
	y = textRect.y + textRect.height;
	return y;
}
/**
 * find the first visible node whose yPosition
 * is greater than the y coordinate of the given
 * point, and return it.
 */
RendererNode find(Point pt) {
	if (pt.y < textRect.y)  {
		return null;
	}
	if (pt.y <= (textRect.y + textRect.height))  {
		return this;
	}
	return null; 
}
abstract String getHintText();
protected Rectangle getImageRect() {
	return imageRect;
}
abstract PopupMenu getPopupMenu(Renderer renderer);
abstract Menu getSelectedMenu(Renderer renderer);
int getWidth() {
	return textRect.x + textRect.width;
}
RendererNode nodeAtIndex(int i) {
	return 0 == i ? this : null;
}
public boolean paste() {return false;}
public void recalculate(boolean recurse) {
}
public final void select(boolean s) {
	selected = s;
}
/**
 * return the height in pixels of one line of entry.
 */
static int UnitHeight(Graphics g) {
	FontMetrics fm = g.getFontMetrics();
	int h = 2 * VERT_MARGIN;
	h += fm.getMaxAscent();
	return h;
}
}
