package com.ibm.uia.renderer;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * This type was created in VisualAge.
 */
public class TreePanel extends ComponentPanel {
/**
 * RenderPanel constructor comment.
 */
public TreePanel(RendererPanel renderer) {
	super(renderer);
}
private final RendererTreeNode _getNodeForObject(RenderableObject ro) {
	Enumeration enum = rootNodes.elements();
	while (enum.hasMoreElements()) {
		RendererTreeNode rtn = (RendererTreeNode)enum.nextElement();
		if (rtn.getRenderableObject().equals(ro)) {
			return rtn;
		}
	}
	return null;
}
final boolean _setSelectedNode(RendererNode select) {
	super._setSelectedNode(select);
	rendererPanel.treeSelected(select);
	return true;
}
public void keyPressed(KeyEvent e) {
	int key = e.getKeyCode();
	int mod = e.getModifiers();
	switch (key) {
		case e.VK_ENTER :
			if (null != selectedNode) {
				if (e.SHIFT_MASK == mod) {
					PopupMenu popup = selectedNode.getPopupMenu(getRenderer());
					if (null != popup) {
						this.add(popup);
						Rectangle rec = selectedNode.getImageRect();
						popup.show(this, rec.x, rec.y);
					}
				} else {
					toggleExpansion((RendererTreeNode)selectedNode);
				}
			}
			break;
		case e.VK_DELETE :
			if (selectedNode.delete()) {
				recalculate(true);
				repaint();
			}
			break;
		default :
			super.keyPressed(e);
			break;
	}
}
/**
 * mouseClicked method comment.
 */
public void mouseClicked(MouseEvent e) {
	RendererTreeNode rtn = (RendererTreeNode)_getNode(e.getPoint());
	if (null != rtn 
			&& e.getClickCount() > 1 
				&& rtn.isHotspot(e.getPoint())) {
		toggleExpansion(rtn);
	} else {
		super.mouseClicked(e);
	}
}
/**
 * mousePressed method comment.
 */
public void mousePressed(MouseEvent e) {
	if (!isPopupTrigger(e)) {
		Point pt = e.getPoint();
		RendererTreeNode rtn = (RendererTreeNode)_getNode(pt);
		if (null != rtn) {
			if (rtn.isHotspot(pt)) {
				toggleExpansion(rtn);
				return;
			}
		}
	}
	super.mousePressed(e);
}
/**
 * create and return an initialised instance of RootTreeNode
 * for the receiver to use as a child.
 * This has been exposed to subclasses so that they can override
 * and substitute their own tree nodes.
 */
protected RendererNode newRootNode(RenderableObject rob) {
	return new RendererTreeNode(rob, null);
}
public void paint(Graphics g) {
	//super.paint(g);
	FontMetrics fmet = g.getFontMetrics();
	height = 0;
	maxWidth = 0;
	Enumeration enum = rootNodes.elements();
	while (enum.hasMoreElements()) {
		RendererTreeNode rtn = (RendererTreeNode)enum.nextElement();
		int w = rtn.getWidth();
		if (w > maxWidth) {
			maxWidth = w;
		}
		height = rtn.draw(g, height, true, this);
	}
}
final void recalculate(boolean recurse) {
	Vector newChildren = new Vector();
	Enumeration rootEnum = getRootObjects();
	while (rootEnum.hasMoreElements()) {
		RenderableObject root = (RenderableObject) rootEnum.nextElement();
		Enumeration objectEnum = root.getAllChildren();
		while (objectEnum.hasMoreElements()) {
			// itterate through real children of object
			RenderableObject ro = (RenderableObject) objectEnum.nextElement();
			// find matching node
			RendererNode rtn = _getNodeForObject(ro);
			if (null == rtn) {
				// create a matching node
				rtn = newRootNode(ro);
			}
			newChildren.addElement(rtn);
			if (recurse) {
				rtn.recalculate(recurse);
			}
		}
	}
	rootNodes = newChildren;
}
private final boolean toggleExpansion(RendererTreeNode rtn) {
	rtn.setExpanded(!rtn.getExpanded());
	if (!rtn.getExpanded()) {
		if (rtn.isSelectedOrChildSelected()) {
			_setSelectedNode(rtn);
		}
	}
	repaint();
	return false;
}
}
